/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip.patterns;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.se.eip.ExchangeContext;
import org.ow2.petals.se.eip.async.CommonAsyncContext;

/**
 * A default AbstractMEPBridgePattern for a pattern with only one Consumes invoked
 * 
 * @author Frederic Gardes
 */
public abstract class AbstractMEPBridgeWithUnicConsumesPattern extends AbstractMEPBridgePattern {

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.se.eip.patterns.AbstractPattern#processAsync(org.ow2.petals.component.framework.api.message.Exchange, org.ow2.petals.se.eip.ExchangeContext, org.ow2.petals.se.eip.async.CommonAsyncContext)
     */
    @Override
    public boolean processAsync(final Exchange exchange, final ExchangeContext context, final CommonAsyncContext asyncContext) {
        try {
            final int subMessages = asyncContext.decrementAndGetOngoingSubMessages();
            if (subMessages == 0) {
                this.processResponse(exchange, context, asyncContext);
            } else {
                if (!asyncContext.isTerminated()) {
                    asyncContext.getOriginalExchange().setError(new Exception("Terminate the pattern processing due to a bad global routing"));
                    context.send(asyncContext.getOriginalExchange());
                    asyncContext.setTerminated();
                }
            }
        } catch (final MessagingException e) {
            this.logger.warning(this.getName() + ": unable to close the source exchange on error status");
        }
        return false;
    }
}
